# -*- coding: utf-8 -*-
#
# WebFilterAdmin a web configurator for dansguardian & squid.
# Copyright (C) 2005 Marco Andreini
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""

Wrapper for Services

"""

__revision__ = 0.5
__author__ = "Marco Andreini"

import os
from popen2 import Popen4


systemService = "system"


class BaseService(object):
    """base service"""

    def __init__(self, name, process=None):
        self.name = name
        self.process = process or name

    def getName(self):
        """service name"""

        return self.name

    def externalCommand(self, cmdline):
        """
        run external command (cmdline)

        return (result, data)
        """

        # FIXME: timer?
        popen = Popen4(cmdline)
        popen.tochild.close()
        
        # FIXME: oh no! one big buffer!
        data = popen.fromchild.read()
        result = popen.wait()
        if os.WIFEXITED(result):
            return (os.WEXITSTATUS(result), data)
        return (result, "")

    def running(self):
        """running"""

        return self.externalCommand("pgrep -o %s" % (self.process, ))[0] == 0

    def serviceControl(self, action):
        """run (via sudo) /etc/init.d/service action"""

        if action not in ("start", "stop", "reload", "restart"):
            raise ValueError, "invalid action"
        
        return self.externalCommand("sudo /etc/init.d/%s %s" % \
                                    (self.name, action))[0] == 0


class SystemService(BaseService):
    """system: a virtual service"""

    def running(self):
        """always True"""
        
        return True

    def serviceControl(self, action):
        """run (via sudo) shutdown for stop"""

        if action == "stop":
            return self.externalCommand("sudo /sbin/shutdown -h now")[0] == 0
        elif action == "reload":
            return self.externalCommand("sudo /sbin/shutdown -r now")[0] == 0
        else:
            raise ValueError, "invalid action"


def Service(name):
    """make service"""

    if name == systemService:
        return SystemService(systemService)
    elif name == "postgresql":
        return BaseService(name, "postmaster")
    return BaseService(name)
